/************************************************************************
    Work/Break Cycle Timer
    Original Copyright (C) 2009  by  David Wilson
    Modifications Copywright 2010 by DJB
        The goal here is to set the first interval for "work time" (white), second interval for the "alarm time" (red), and the third interval for the "break time" (blue)
        It will count down from the first, alarm goes off for X seconds, and then count down from the third, cycling until stopped.

        There have been too many changes to the original source code to list them all.
*************************************************************************/

var displayColor;
var dimmedColor = "#606060";
var colorTable = new Array("FFFFFF", "FF5050", "87CEFA", "90EE90", "FF70FF", "FFA07A", "E8E850");

var Hours = 2;
var Minutes = 1;
var Seconds = 0;
var defaultMinutes0 = 15;
var defaultSeconds1 = 5;
var defaultMinutes2 = 3;

var displayTimer;                   // Which timer is being shown, 0 or 1.
var timerFunction = new Array("onTimer(0)", "onTimer(1)", "onTimer(2)"); // To be called by setInterval.
var timerStatus = 0;                // Status of the timers, inactive (0), active (1), or ended (2)
var timerInterval = 0;              // How long the interval should be.
var timerCount = new Array(3);      // Holds the times for the timers
timerCount[0] = new Array(3);       // counts for 1st timer: 2 = hrs count, 1 = mins count, 0 = secs count
timerCount[1] = new Array(3);       // counts for 2nd timer: 2 = hrs count, 1 = mins count, 0 = secs count
timerCount[2] = new Array(3);       // counts for 2nd timer: 2 = hrs count, 1 = mins count, 0 = secs count
var timerCycle = timerCount.slice();      // Holds the times that will be restored when the present cycle finishes.
var previousTimer = 0;
var halfSecond = false;
var baseCount;
var basey;

System.Gadget.settingsUI = "qtimer_settings.html";

System.Gadget.onSettingsClosed = SettingsClosed;

// - DB: Special function to copy an object but not have it referenced.
Object.prototype.clone = function () {
    var newObj = (this instanceof Array) ? [] : {};
    for (i in this) {
        if (i == 'clone') continue;
        if (this[i] && typeof this[i] == "object") {
            newObj[i] = this[i].clone();
        } else newObj[i] = this[i]
    } return newObj;
};

function onLoad() {
    var alarmIndex, alarmSound, color0Index, color1Index, color2Index, color3Index, color4Index;

    alarmIndex = System.Gadget.Settings.read("alarmIndex");
    if (alarmIndex == "") {
        alarmIndex = 0;
        System.Gadget.Settings.write("alarmIndex", alarmIndex);
    }

    alarmSound = System.Gadget.Settings.read("alarmSound");
    if (alarmSound == "") {
        alarmSound = "chimes.wav";
        System.Gadget.Settings.write("alarmSound", alarmSound);
    }

    color0Index = System.Gadget.Settings.read("color0Index");
    if (color0Index == "") {
        color0Index = 0;
        System.Gadget.Settings.write("color0Index", color0Index);
    }

    color1Index = System.Gadget.Settings.read("color1Index");
    if (color1Index == "") {
        color1Index = 1;
        System.Gadget.Settings.write("color1Index", color1Index);
    }

    color2Index = System.Gadget.Settings.read("color2Index");
    if (color2Index == "") {
        color2Index = 2;
        System.Gadget.Settings.write("color2Index", color2Index);
    }

    displayColor = colorTable[color0Index];

    startButton.style.backgroundColor = displayColor;
    hours.style.color = dimmedColor;
    minutes.style.color = dimmedColor;
    seconds.style.color = dimmedColor;

    timerCount[0][Hours] = 0;
    timerCount[0][Minutes] = defaultMinutes0;
    timerCount[0][Seconds] = 0;

    timerCount[1][Hours] = 0;
    timerCount[1][Minutes] = 0;
    timerCount[1][Seconds] = defaultSeconds1;

    timerCount[2][Hours] = 0;
    timerCount[2][Minutes] = defaultMinutes2;
    timerCount[2][Seconds] = 0;

    displayTimer = 0;

    updateDisplay();
    updateColor();
}

function onUnload() {
}

function SettingsClosed(event) {
    updateColor();
}

function updateColor() {
    var colorIndex;

    colorIndex = System.Gadget.Settings.read("color" + displayTimer + "Index");

    displayColor = colorTable[colorIndex];

    if (hours.style.color != dimmedColor) hours.style.color = displayColor;
    if (minutes.style.color != dimmedColor) minutes.style.color = displayColor;
    if (seconds.style.color != dimmedColor) seconds.style.color = displayColor;

    otherButton.style.backgroundColor = displayColor;
    if (startButton.style.backgroundColor != dimmedColor) startButton.style.backgroundColor = displayColor;
    resetButton.style.backgroundColor = displayColor;
}

// Called every second for active timers by setInterval.
function onTimer(timer) {
    var alarmSound;

    if (timerStatus >= 1) countdownTimer(timer);

    if (timerStatus == 2) 
        timerStatus = 3;
    else if (timerStatus == 3) {
        timerStatus = 2;
        alarmSound = System.Gadget.Settings.read("alarmSound");
        System.Sound.playSound(alarmSound);
    }
    if (timer == displayTimer) updateDisplay();
}

// This keeps track for time and increments the time down by one second. If it's at zero, it would trigger an elevation of state.
function countdownTimer(timer) {
    // Special handing for the alarm timer, it gets two ticks per second:
    if (timer == 1) {
        if (!halfSecond) halfSecond = true;
        else {
            halfSecond = false;
            timerCount[timer][Seconds] = timerCount[timer][Seconds] - 1;
        }
    }
    else
        timerCount[timer][Seconds] = timerCount[timer][Seconds] - 1;

    if (timerCount[timer][Seconds] < 0) {
        timerCount[timer][Seconds] = 59;
        timerCount[timer][Minutes] = timerCount[timer][Minutes] - 1;
        if (timerCount[timer][Minutes] < 0) {
            timerCount[timer][Minutes] = 59;
            timerCount[timer][Hours] = timerCount[timer][Hours] - 1;
            if (timerCount[timer][Hours] < 0) {
                timerCount[timer][Hours] = 23;
            }
        }
    }
    // Reached the end? What it does next is highly important:
    if (timerCount[timer][Hours] == 0 && timerCount[timer][Minutes] == 0 && timerCount[timer][Seconds] == 0) {
        clearInterval(timerInterval);
        timerCount[timer] = timerCycle[timer].slice(); // Restore it to its starting point.

        // Intermediate Timer:
        if (timer == 1) {
            displayTimer = 2 - previousTimer;
            previousTimer = displayTimer;
            timerStatus = 1;
        }
        // Study/Break Timer:
        else {
            displayTimer = 1;
            timerStatus = 3;
        }

        // Initiate:
        timerCycle[displayTimer] = timerCount[displayTimer].slice(); // Record starting position - it may have been adjusted.
        timerInterval = setInterval(timerFunction[displayTimer], displayTimer == 1 ? 500 : 1000);

        updateButton();
        updateDisplay();
        updateColor();
    }
}

function updateDisplay() {
    if (timerCount[displayTimer][Hours] < 10) hours.innerText = "0" + timerCount[displayTimer][Hours];
    else hours.innerText = timerCount[displayTimer][Hours];
    if (timerCount[displayTimer][Minutes] < 10) minutes.innerText = "0" + timerCount[displayTimer][Minutes];
    else minutes.innerText = timerCount[displayTimer][Minutes];
    if (timerCount[displayTimer][Seconds] < 10) seconds.innerText = "0" + timerCount[displayTimer][Seconds];
    else seconds.innerText = timerCount[displayTimer][Seconds];

    if (timerStatus <= 1) {
        startButton.style.backgroundColor = displayColor;
        hours.style.color = displayColor;
        minutes.style.color = displayColor;
        seconds.style.color = displayColor;
        if (timerCount[displayTimer][Hours] == 0) hours.style.color = dimmedColor;
        if (timerCount[displayTimer][Hours] == 0 && timerCount[displayTimer][Minutes] == 0) minutes.style.color = dimmedColor;
        if (timerCount[displayTimer][Hours] == 0 && timerCount[displayTimer][Minutes] == 0 && timerCount[displayTimer][Seconds] == 0) seconds.style.color = dimmedColor;
    }
    // Colors for when a timer is going off:
    else if (timerStatus >= 2) {
        // Only dim for 
        if (timerStatus == 2 && displayTimer == 1) {
            startButton.style.backgroundColor = dimmedColor;
            hours.style.color = dimmedColor;
            minutes.style.color = dimmedColor;
            seconds.style.color = dimmedColor;
        }
        else {
            startButton.style.backgroundColor = displayColor;
            hours.style.color = displayColor;
            minutes.style.color = displayColor;
            seconds.style.color = displayColor;
        }
    }
}

function updateButton() {
    if (timerStatus == 0) startbtnimg.src = "Images/startbtn.png";
    else if (timerStatus == 1) startbtnimg.src = "Images/pausebtn.png";
    else if (timerStatus >= 2) startbtnimg.src = "Images/alarmbtn.png";
}

// - DB: This is the switch clock button:
function otherTimer() {
    displayTimer = displayTimer == (2) ? 0 : displayTimer + 1;
    otherbtnimg.src = "Images/otherbtn" + displayTimer + ".png";

    updateDisplay();
    updateColor();
}

// - DB: This is the start/pause/stop button:
function startTimer() {
    // Triggered when start button is pressed:
    if (timerStatus == 0) {
        if (timerCount[displayTimer][Hours] != 0 || timerCount[displayTimer][Minutes] != 0 || timerCount[displayTimer][Seconds] != 0) {

            if (displayTimer == 1) displayTimer = 0;
            previousTimer = displayTimer;
            timerCycle = timerCount.clone();

            timerStatus = 1;
            timerInterval = setInterval(timerFunction[displayTimer], 1000);
        }
    }
    // Triggered when pause button is pressed:
    else if (timerStatus == 1) {
        timerStatus = 0;
        clearInterval(timerInterval);
    }
    // Triggered when the stop button is pressed (timer is going off)
    else {
        timerStatus = 0;
        clearInterval(timerInterval);
        timerCount[displayTimer] = timerCycle[displayTimer].slice(); // Restore it to its starting point.
    }

    updateButton();
    updateDisplay();
    updateColor();
}

function resetTimer() {
    if (timerStatus >= 1) {
        timerStatus = 0;
        clearInterval(timerInterval);
    }

    // DB:
    if (displayTimer == 0) {
        timerCount[displayTimer][Hours] = 0;
        timerCount[displayTimer][Minutes] = defaultMinutes0;
        timerCount[displayTimer][Seconds] = 0;
    }
    else if (displayTimer == 1) {
        timerCount[displayTimer][Hours] = 0;
        timerCount[displayTimer][Minutes] = 0;
        timerCount[displayTimer][Seconds] = defaultSeconds1;
    }
    else {
        timerCount[displayTimer][Hours] = 0;
        timerCount[displayTimer][Minutes] = defaultMinutes2;
        timerCount[displayTimer][Seconds] = 0;
    }

    updateButton();
    updateDisplay();
}

function setTimer(field) {
    var hms;

    if (field == hours) hms = Hours;
    else if (field == minutes) hms = Minutes;
    else if (field == seconds) hms = Seconds;

    field.setCapture();

    baseCount = timerCount[displayTimer][hms];
    basey = event.y;

    field.onmousemove =
        function () {
            timerCount[displayTimer][hms] = baseCount + Math.floor((basey - event.y) / 4);
            if (timerCount[displayTimer][hms] < 0) timerCount[displayTimer][hms] = 0;
            else if (hms == Hours && timerCount[displayTimer][hms] > 23) timerCount[displayTimer][hms] = 23;
            else if (timerCount[displayTimer][hms] > 59) timerCount[displayTimer][hms] = 59;
            updateDisplay();
        }

    field.onmouseup =
        function () {
            field.releaseCapture();
            field.onmousemove = null;
            field.onmouseup = null;
            timerCount[displayTimer][hms] = baseCount + Math.floor((basey - event.y) / 4);
            if (timerCount[displayTimer][hms] < 0) timerCount[displayTimer][hms] = 0;
            else if (hms == Hours && timerCount[displayTimer][hms] > 23) timerCount[displayTimer][hms] = 23;
            else if (timerCount[displayTimer][hms] > 59) timerCount[displayTimer][hms] = 59;
            updateDisplay();
        }
}    

